/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package global.sandbox.xmlutilities;

import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import javax.xml.transform.Source;
import junit.framework.TestCase;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 *
 * @author Petr Hadraba
 */
public class XMLUtilitiesUsageTest extends TestCase {
    
    private XMLUtilities xmlUtilities;
    
    private NamespaceContextImpl nsCtx;
    
    @Override
    protected void setUp() throws Exception {
        super.setUp();
        
        // we should use default objects, no customizations
        this.xmlUtilities = new XMLUtilities();
        this.nsCtx = createNamespaceContextImpl();
    }
    
    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
        
        this.nsCtx = null;
        this.xmlUtilities = null;
    }
    
    /**
     * demonstrates validation
     * 
     * @throws java.lang.Exception
     */
    public void testValidateValidDocument() throws Exception {
        Document schema = loadSchema();
        Document validXML = loadDocument("valid.xml");
        
        String result = xmlUtilities.validateXmlUsingSchema(
                XMLTools.documentToDOMSource(validXML),
                XMLTools.documentToDOMSource(schema));
        
        System.err.println("The resulting String is set to null if validation"
                + " is OK: " + result);
        
        assertNull(result);
    }
    
    /**
     * demonstrates validation
     * 
     * @throws java.lang.Exception
     */
    public void testValidateInvalidDocument() throws Exception {
        Document schema = loadSchema();
        Document invalidXML = loadDocument("invalid.xml");
        
        String result = xmlUtilities.validateXmlUsingSchema(
                XMLTools.documentToDOMSource(invalidXML),
                XMLTools.documentToDOMSource(schema));
        
        System.err.println("The resulting String contains error message"
                + " if problem found during validation process: " + result);
        
        assertNotNull(result);
    }
    
    /**
     * demonstrates XPath evaluation with namespaces
     * 
     * @throws java.lang.Exception
     */
    public void testEvaluateXPath_1() throws Exception {
        Document validXML = loadDocument("valid.xml");
        
        NodeList elements = xmlUtilities.evaluateXPath(
                "ns:properties/ns:property",
                validXML,
                nsCtx);
        
        assertEquals(
                2,
                elements.getLength());
    }
    
    /**
     * demonstrates XPath evaluation with namespaces
     * 
     * @throws java.lang.Exception
     */
    public void testEvaluateXPath_2() throws Exception {
        Document validXML = loadDocument("valid.xml");
        
        Node attribute = xmlUtilities.getFirstNodeForXPath(
                "ns:properties/ns:property/@name",
                validXML,
                nsCtx);
        
        assertEquals(
                Node.ATTRIBUTE_NODE,
                attribute.getNodeType());
        
        assertEquals(
                "p1",
                ((Attr) attribute).getValue());
    }
    
    /**
     * demonstrates XSL transformation
     * 
     * @throws java.lang.Exception
     */
    public void testTransformation() throws Exception {
        Source validXML = XMLTools.documentToDOMSource(loadDocument("valid.xml"));
        Source style = XMLTools.documentToDOMSource(loadDocument("style.xsl"));
        
        String result = xmlUtilities.transformToString(
                style,
                validXML);
        
        System.err.println("Transformed properties: " + result);
        
        assertNotNull(result);
    }
    
    /**
     * demonstrates XSL transformation with parameters
     * 
     * @throws java.lang.Exception
     */
    public void testTransformationWithParameters() throws Exception {
        Source validXML = XMLTools.documentToDOMSource(loadDocument("valid.xml"));
        Source style = XMLTools.documentToDOMSource(loadDocument("parametrized-style.xsl"));
        
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put("parameter", "Value of the parameter");
        
        String result = xmlUtilities.transformToString(
                style,
                validXML,
                parameters);
        
        System.err.println("Transformed properties: " + result);
        
        assertNotNull(result);
    }
    
    /* HELPER METHODS */
    
    private Document loadSchema() throws Exception {
        return loadDocument("sample.xsd");
    }
    
    private Document loadDocument(String name) throws Exception {
        InputStream is = getClass().getResourceAsStream(name);
        
        try {
            return xmlUtilities.loadDocumentFromStream(is);
        } finally {
            is.close();
        }
    }
    
    private NamespaceContextImpl createNamespaceContextImpl() throws Exception {
        NamespaceContextImpl nsctx = new NamespaceContextImpl();
        
        nsctx.addNamespace("ns", "http://hadrabap.googlepages.com/projects/"
                + "xmlutilities/sample.xsd");
        
        return nsctx;
    }
    
}
